@@ -12,6 +12,9 @@ setClass("GenomicRanges",
         #No more constraint slot for now...
         #constraint="Constraint_OR_NULL",
         elementMetadata="DataFrame"
+    ),
+    prototype(
+        elementMetadata=new("DFrame")
     )
 )
 
@@ -32,7 +35,8 @@ setClass("GenomicPos",
 ###
 
 ### The method for Ranges does 'length(start(x))' which is very inefficient
-### on GPos objects so we overwrite it with a faster method.
+### on a StitchedGPos object so we overwrite it with a method that should be
+### fast on all GenomicRanges derivatives (including StitchedGPos objects).
 setMethod("length", "GenomicRanges", function(x) length(ranges(x)))
 
 setMethod("names", "GenomicRanges", function(x) names(ranges(x)))
@@ -148,8 +152,9 @@ make_out_of_bound_warning_msg <- function(x, idx, suggest.trim)
 
 .valid.GenomicRanges.ranges <- function(x)
 {
-    if (!(class(ranges(x)) %in% c("IRanges", "IPos")))
-        return("'ranges(x)' must be an IRanges or IPos instance")
+    if (!(class(ranges(x)) %in% c("IRanges", "StitchedIPos", "UnstitchedIPos")))
+        return(paste0("'ranges(x)' must be an IRanges, StitchedIPos, ",
+                      "or UnstitchedIPos instance"))
     NULL
 }
 
@@ -172,6 +177,8 @@ make_out_of_bound_warning_msg <- function(x, idx, suggest.trim)
 ### Keep the 2 lists in sync!
 ### We don't put "genome" in that list in order to facilitate import of GFF3
 ### files as GRanges objects (see ?import.gff3 in rtracklayer).
+### TODO: Get rid of this restriction. Not sure why we ever had it. Doesn't
+### make much sense to me.
 INVALID.GR.COLNAMES <- c("seqnames", "ranges", "strand",
                          "seqlevels", "seqlengths", "isCircular",
                          #"genome",
@@ -218,7 +225,7 @@ setValidity2("GenomicRanges", .valid.GenomicRanges)
 
 
 ### - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-### Coercion.
+### Coercion
 ###
 
 ### Propagate the names.
@@ -257,7 +264,6 @@ setAs("GenomicRanges", "Grouping", function(from) {
 setMethod("as.data.frame", "GenomicRanges",
     function(x, row.names=NULL, optional=FALSE, ...)
     {
-        ranges <- ranges(x)
         if (missing(row.names))
             row.names <- names(x)
         if (!is.null(names(x)))
@@ -388,20 +394,20 @@ set_GenomicRanges_seqinfo <-
              pruning.mode=c("error", "coarse", "fine", "tidy"),
              value)
 {
-    pruning.mode <- match.arg(pruning.mode)
-    if (pruning.mode == "fine")
-        stop(wmsg("\"fine\" pruning mode not supported on ",
-                  class(x), " objects"))
     if (!is(value, "Seqinfo"))
         stop("the supplied 'seqinfo' must be a Seqinfo object")
+    pruning.mode <- match.arg(pruning.mode)
+    if (pruning.mode == "fine")
+        stop(wmsg("\"fine\" pruning mode is not supported on ",
+                  class(x), " objects"))
     dangling_seqlevels <- GenomeInfoDb:::getDanglingSeqlevels(x,
                               new2old=new2old,
                               pruning.mode=pruning.mode,
                               seqlevels(value))
     if (length(dangling_seqlevels) != 0L) {
         ## Prune 'x'.
-        non_dangling_range <- !(seqnames(x) %in% dangling_seqlevels)
-        x <- x[non_dangling_range]
+        idx <- !(seqnames(x) %in% dangling_seqlevels)
+        x <- x[idx]
     }
     old_seqinfo <- seqinfo(x)
     new_seqnames <- GenomeInfoDb:::makeNewSeqnames(x,
@@ -418,13 +424,28 @@ set_GenomicRanges_seqinfo <-
 }
 setReplaceMethod("seqinfo", "GenomicRanges", set_GenomicRanges_seqinfo)
 
-setMethod("score", "GenomicRanges", function(x) mcols(x, use.names=FALSE)$score)
-setReplaceMethod("score", "GenomicRanges", function(x, value) {
-  ## Fix old GRanges instances on-the-fly.
-  x <- updateObject(x)
-  mcols(x)$score <- value
-  x
-})
+
+### - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
+### The score() accessor
+###
+### Kind of a silly accessor. And why is it defined at the GenomicRanges
+### level and not at the Vector level? Or at least at the Ranges and
+### RangesList levels so it works on IRanges and IRangesList objects too.
+###
+
+setMethod("score", "GenomicRanges",
+    function(x) mcols(x, use.names=FALSE)$score
+)
+
+setReplaceMethod("score", "GenomicRanges",
+    function(x, value)
+    {
+        ## Fix old GRanges instances on-the-fly.
+        x <- updateObject(x)
+        mcols(x)$score <- value
+        x
+    }
+)
 
 #setReplaceMethod("constraint", "GenomicRanges",
 #    function(x, value)
@@ -635,7 +656,7 @@ summary.GenomicRanges <- function(object, ...)
     .GenomicRanges_summary(object, ...)
 setMethod("summary", "GenomicRanges", summary.GenomicRanges)
 
-.make_naked_matrix_from_GenomicRanges <- function(x)
+.from_GenomicRanges_to_naked_character_matrix_for_display <- function(x)
 {
     x_len <- length(x)
     x_mcols <- mcols(x, use.names=FALSE)
@@ -665,7 +686,7 @@ show_GenomicRanges <- function(x, margin="",
                                print.classinfo=FALSE, print.seqinfo=FALSE,
                                coerce.internally.to.GRanges=TRUE)
 {
-    cat(.GenomicRanges_summary(x), ":\n", sep="")
+    cat(margin, summary(x), ":\n", sep="")
     ## S4Vectors:::makePrettyMatrixForCompactPrinting() assumes that head()
     ## and tail() work on 'xx'.
     if (coerce.internally.to.GRanges) {
@@ -674,7 +695,7 @@ show_GenomicRanges <- function(x, margin="",
         xx <- x
     }
     out <- S4Vectors:::makePrettyMatrixForCompactPrinting(xx,
-                .make_naked_matrix_from_GenomicRanges)
+                .from_GenomicRanges_to_naked_character_matrix_for_display)
     if (print.classinfo) {
         .COL2CLASS <- c(
             seqnames="Rle",
@@ -690,21 +711,20 @@ show_GenomicRanges <- function(x, margin="",
         out <- rbind(classinfo, out)
     }
     if (nrow(out) != 0L)
-        rownames(out) <- paste0(margin, rownames(out))
+        rownames(out) <- paste0(margin, "  ", rownames(out))
     ## We set 'max' to 'length(out)' to avoid the getOption("max.print")
     ## limit that would typically be reached when 'showHeadLines' global
     ## option is set to Inf.
     print(out, quote=FALSE, right=TRUE, max=length(out))
     if (print.seqinfo) {
-        cat(margin, "-------\n", sep="")
-        cat(margin, "seqinfo: ", summary(seqinfo(x)), "\n", sep="")
+        cat(margin, "  -------\n", sep="")
+        cat(margin, "  seqinfo: ", summary(seqinfo(x)), "\n", sep="")
     }
 }
 
 setMethod("show", "GenomicRanges",
     function(object)
-        show_GenomicRanges(object, margin="  ",
-                           print.classinfo=TRUE, print.seqinfo=TRUE)
+        show_GenomicRanges(object, print.classinfo=TRUE, print.seqinfo=TRUE)
 )
 
 
@@ -712,6 +732,20 @@ setMethod("show", "GenomicRanges",
 ### Concatenation
 ###
 
+### 'x' can be any list-like object and its list elements can be anything
+### that supports seqinfo().
+### Used in the seqinfo() method for GenomicRangesList objects.
+combine_seqinfo_from_GenomicRanges_objects <- function(x)
+{
+    x_len <- length(x)
+    ## This will typically happen in the context of the seqinfo() method
+    ## for GenomicRangesList objects.
+    if (x_len == 0L)
+        stop(wmsg("seqinfo() is not supported on a zero length ",
+                  class(x), " object"))
+    do.call(merge, lapply(seq_len(x_len), function(i) seqinfo(x[[i]])))
+}
+
 ### NOT exported but used in the GenomicAlignments package.
 concatenate_GenomicRanges_objects <-
     function(x, objects=list(), use.names=TRUE, ignore.mcols=FALSE, check=TRUE)
@@ -723,7 +757,7 @@ concatenate_GenomicRanges_objects <-
     all_objects <- c(list(x), objects)
 
     ## Combine seqinfo.
-    seqinfo(x) <- do.call(merge, lapply(all_objects, seqinfo))
+    seqinfo(x) <- combine_seqinfo_from_GenomicRanges_objects(all_objects)
 
     ## Call method for Vector objects to concatenate all the parallel slots.
     callNextMethod()
